function [LogMargs,CovProb] = MargDensModHarmonic(PostSample,LogPost,CovProb,Origin,StepSize,Chain)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE:     Computes the marginal likelihood from a MCMC sample using the Modified Harmonic Mean estimator 
%              (Geweke, 1999, Econometric Reviews). An estimate is computed sequentially over the posterior sample 
%              (at the MCMC iterations Origin:StepSize:Niter) to assess convergence via CUSUM-type plots. 
%              CUMSUM-type plots are produced if the log marginal likelihood is estimated sequentially.
%              Optional input arguments are enclosed in brackets. Enter them as empty ([]) matrices in the function call.
%
% INPUT:       PostSample      (niter-by-npara)    Posterior draws on all model parameters
%              LogPost         (niter-by-1)        Posterior density at each PostSample draw
%              [CovProb]       (NCovProb-by-1)     Coverage probability (ellipsoid) for the harmonic estimator. Default CovProb=0.1:0.1:0.9
%              [Origin]        (Scalar)            First draw of the posterior sample where the marginal likelihood is computed
%              [StepSize]      (Scalar)            Stepsize of the sequenial marginal likelihood etimates
%              [Chain]         (Scalar)            Should be Chain=1 unless used in a multiple chain setting.
%
% OUTPUT:      LogMargs        (NSeq-by-NCovProb)  Matrix with sequential estimates of the log marginal likelihood. 
%                                                  LogMargs(:,i) contains the sequential estimates of the i:th coverage probability.
%              CovProb         (NCovProb-by-1)     Vector with the coverage probabilities. Same as the input CovProb.          
%
% AUTHOR:      Mattias Villani, Research Department, Sveriges Riksbank and
%              Department of Statistics, Stockholm University. 
%              E-mail: mattias.villani@riksbank.se
%
% REVISED:     2004-03-14
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Default settings and initiailization %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isempty(CovProb)
    CovProb=0.1:0.1:0.9;
end
CovProb=CovProb(:)'; % Making sure the CovProb is a row vector.
LogMargs=[];
[NFull,NPara]=size(PostSample);

if isempty(Origin) & isempty(StepSize)==0
    Origin=StepSize;
end

if isempty(Origin)==0 & isempty(StepSize)
    StepSize=round((NFull-Origin)/10);
end

if isempty(Origin) & isempty(StepSize)
    Origin=NFull;
    StepSize=NFull;
end

SeqGrid=Origin:StepSize:NFull;
if isempty(SeqGrid)
    LogMargs=nan;
    CovProb=nan;
    return
end

if SeqGrid(length(SeqGrid))<NFull
    SeqGrid=[SeqGrid NFull];
end

if isempty(Chain)
    Chain==1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Begin sequential estimation of marginal likelihood %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SumXX = zeros(NPara,NPara);
Sigma= zeros(NPara,NPara);
MaxLogPost = max(LogPost);
mu=zeros(NPara,1);
SumX=zeros(NPara,1);
StartEst=1;
for N=SeqGrid
    
    % Computing the posterior mean and covariance matrix
    SigmaIsOK=1;
    %mu = mean(PostSample(1:N,:))'
    SumX=SumX+sum(PostSample(StartEst:N,:))';
    mu=SumX/N;
    SumXX=SumXX+PostSample(StartEst:N,:)'*PostSample(StartEst:N,:);
    Sigma=SumXX/N-mu*mu';
    
    %LogDetSigma = log(det(Sigma));
    LogDetSigma=sum(log(eig(Sigma)));
    InvSigma = inv(Sigma);
    
    LogMarg = [];
    % Checking if Sigma is ill-behaved
    if isinf(LogDetSigma) | isnan(LogDetSigma) | isreal(LogDetSigma)==0
        LogMargs=[LogMargs;nan*ones(1,length(CovProb))];
        SigmaIsOK=0;
    end

    % Marginal likelihood, M(i), computed by the formula 1/M(i)=E[f(theta)/(L(theta)*Prior(theta))],
    % where E is posterior expectation, f is a ellipsoid truncted multivariate normal density with coverage probability p.
    if SigmaIsOK
        for p = CovProb
            critval = qchisq(p,NPara);
            Temp=0;
            for i = 1:N
                deviation  = (PostSample(i,:)-mu')*InvSigma*((PostSample(i,:)-mu'))';
                if deviation <= critval;
                    TruncNormalDens = -log(p)-(NPara*log(2*pi)+LogDetSigma+deviation)/2; % f-density evaluated at ith posterior draw.
                    Temp = Temp + exp(TruncNormalDens - LogPost(i)+MaxLogPost); % Adding maximum of log posterior to bring down the scale before taking exp().
                end
            end     
            LogMarg=[LogMarg;p MaxLogPost-log(Temp/N)];
        end
        LogMargs=[LogMargs;LogMarg(:,2)'];
    end
    StartEst=N+1;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    Plotting the sequential estimates  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if length(SeqGrid)>1
    PlotSymbol={'r--','b.-','g:','yv-','r','b--','g','m','b:'};
    figure('Name',strcat('Sequential log marginal likelihood (ModHarm) for parallel MCMC chain no. ',int2str(Chain)));
    hold on
    for j=1:length(CovProb)
        plot(SeqGrid,LogMargs(:,j),PlotSymbol{j})
    end
    MeanCurve=plot(SeqGrid,mean(LogMargs'),'k');
    set(MeanCurve,'linewidth',1.5)
    for i=1:length(CovProb)
        LegendCell{i}=strcat('p=',num2str(CovProb(i)));
    end
    LegendCell{length(CovProb)+1}='Overall mean';
    legend(LegendCell)
    box off
    ylabel('Log Marginal Likelihood')
    xlabel('Iteration number')
    axis tight
    title('Sequential log marginal likelihood (Modified Harmonic)')
end



